package com.quickblox.quickblox_sdk.webrtc;

import android.content.Context;

import com.quickblox.quickblox_sdk.base.BaseModule;
import com.quickblox.quickblox_sdk.base.EventHandler;
import com.quickblox.videochat.webrtc.QBRTCConfig;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * Created by Injoit on 2021-01-06.
 * Copyright © 2020 Quickblox. All rights reserved.
 */
public class QBRTCConfigModule implements BaseModule {
    static final String CHANNEL_NAME = "FlutterQBRTCConfigChannel";

    private static final String SET_ANSWER_TIME_INTERVAL_METHOD = "setAnswerTimeInterval";
    private static final String GET_ANSWER_TIME_INTERVAL_METHOD = "getAnswerTimeInterval";
    private static final String SET_DIALING_TIME_INTERVAL_METHOD = "setDialingTimeInterval";
    private static final String GET_DIALING_TIME_INTERVAL_METHOD = "getDialingTimeInterval";

    private static final int MIN_ANSWER_TIME_INTERVAL = 10;
    private static final int MIN_DIALING_TIME_INTERVAL = 3;

    private BinaryMessenger binaryMessenger;
    private Context context;

    public QBRTCConfigModule(BinaryMessenger binaryMessenger, Context context) {
        this.binaryMessenger = binaryMessenger;
        this.context = context;

        initEventHandler();
    }

    @Override
    public void initEventHandler() {
        EventHandler.init(QBRTCConfigConstants.getAllEvents(), binaryMessenger);
    }

    @Override
    public String getChannelName() {
        return CHANNEL_NAME;
    }

    @Override
    public MethodChannel.MethodCallHandler getMethodHandler() {
        return this::handleMethod;
    }

    @Override
    public void handleMethod(MethodCall methodCall, MethodChannel.Result result) {
        switch (methodCall.method) {
            case SET_ANSWER_TIME_INTERVAL_METHOD:
                setAnswerTimeInterval(methodCall.arguments(), result);
                break;
            case GET_ANSWER_TIME_INTERVAL_METHOD:
                getAnswerTimeInterval(result);
                break;
            case SET_DIALING_TIME_INTERVAL_METHOD:
                setDialingTimeInterval(methodCall.arguments(), result);
                break;
            case GET_DIALING_TIME_INTERVAL_METHOD:
                getDialingTimeInterval(result);
                break;
        }
    }

    private void setAnswerTimeInterval(int interval, final MethodChannel.Result result) {
        if (interval < MIN_ANSWER_TIME_INTERVAL) {
            result.error("Value should be equal to or greater than " + MIN_ANSWER_TIME_INTERVAL,
                    null, null);
            return;
        }
        QBRTCConfig.setAnswerTimeInterval(interval);
        result.success(null);
    }

    private void getAnswerTimeInterval(final MethodChannel.Result result) {
        long interval = QBRTCConfig.getAnswerTimeInterval();
        result.success(interval);
    }

    private void setDialingTimeInterval(int interval, final MethodChannel.Result result) {
        if (interval < MIN_DIALING_TIME_INTERVAL) {
            result.error("Value should be equal to or greater than " + MIN_DIALING_TIME_INTERVAL,
                    null, null);
            return;
        }
        QBRTCConfig.setDialingTimeInterval(interval);
        result.success(null);
    }

    private void getDialingTimeInterval(final MethodChannel.Result result) {
        long interval = QBRTCConfig.getDialingTimeInterval();
        result.success(interval);
    }
}
